@@ -2,7 +2,7 @@
  * SCIM-chewing -
  *	Intelligent Chinese Phonetic IM Engine for SCIM.
  *
- * Copyright (c) 2004, 2005, 2006
+ * Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010
  *	SCIM-chewing Developers. See ChangeLog for details.
  *
  * See the file "COPYING" for information on usage and redistribution
@@ -46,7 +46,7 @@
 static int _selection_keys_num;
 
 #include <scim.h>
-#include <chewing/chewing.h>
+#include <chewing.h>
 
 #include "scim_chewing_imengine.h"
 #include "scim_chewing_config_entry.h"
@@ -166,15 +166,6 @@ void ChewingIMEngineFactory::reload_config( const ConfigPointer &scim_config )
 		String( SCIM_CONFIG_IMENGINE_CHEWING_USER_KB_TYPE ),
 		String( "KB_DEFAULT" ));
 
-	// Load PinYin method Type
-	m_PinYinType = m_config->read (
-		String( SCIM_CONFIG_IMENGINE_CHEWING_PINYIN_METHD),
-		PINYIN_HANYU);
-
-	m_ExternPinYinPath = m_config->read (
-		String( SCIM_CONFIG_IMENGINE_CHEWING_EXTERNAL_PINYIN_PATH),
-		String(""));
-
 	// SCIM_CONFIG_IMENGINE_CHEWING_USER_SELECTION_KEYS
 	m_selection_keys = m_config->read(
 		String( SCIM_CONFIG_IMENGINE_CHEWING_USER_SELECTION_KEYS ),
@@ -189,7 +180,17 @@ void ChewingIMEngineFactory::reload_config( const ConfigPointer &scim_config )
 	m_add_phrase_forward = m_config->read(
 			String( SCIM_CONFIG_IMENGINE_CHEWING_ADD_PHRASE_FORWARD ),
 			false);
-	
+
+	// SCIM_CONFIG_IMENGINE_CHEWING_PHRASE_CHOICE_REARWARD
+	m_phrase_choice_rearward = m_config->read(
+			String( SCIM_CONFIG_IMENGINE_CHEWING_PHRASE_CHOICE_REARWARD ),
+			true);
+
+	// SCIM_CONFIG_IMENGINE_CHEWING_AUTO_SHIFT_CURSOR
+	m_auto_shift_cursor = m_config->read(
+			String( SCIM_CONFIG_IMENGINE_CHEWING_AUTO_SHIFT_CURSOR ),
+			true);
+
 	// SCIM_CONFIG_IMENGINE_CHEWING_ESC_CLEAN_ALL_BUFFER
 	m_esc_clean_all_buffer = m_config->read(
 			String( SCIM_CONFIG_IMENGINE_CHEWING_ESC_CLEAN_ALL_BUFFER ),
@@ -317,35 +318,34 @@ void ChewingIMEngineInstance::reload_config( const ConfigPointer& scim_config )
 	// Reset all data.
 	reset();
 
-	config.candPerPage = m_factory->m_selection_keys_num;
-
-	config.maxChiSymbolLen = 16;
-
+	chewing_set_candPerPage( ctx, m_factory->m_selection_keys_num );
+	chewing_set_maxChiSymbolLen( ctx, 16 );
 	// SCIM_CONFIG_IMENGINE_CHEWING_ADD_PHRASE_FORWARD
-	config.bAddPhraseForward = m_factory->m_add_phrase_forward ? 0 : 1;
-
+	chewing_set_addPhraseDirection( ctx, m_factory->m_add_phrase_forward ? 0 : 1 );
+	// SCIM_CONFIG_IMENGINE_CHEWING_PHRASE_CHOICE_REARWARD
+	chewing_set_phraseChoiceRearward( ctx, m_factory->m_phrase_choice_rearward ? 1 : 0 );
+	// SCIM_CONFIG_IMENGINE_CHEWING_AUTO_SHIFT_CURSOR
+	chewing_set_autoShiftCur( ctx, m_factory->m_auto_shift_cursor ? 1 : 0 );
 	// SCIM_CONFIG_IMENGINE_CHEWING_SPACE_AS_SELECTION
-	config.bSpaceAsSelection = m_factory->m_space_as_selection ? 1 : 0;
-
+	chewing_set_spaceAsSelection( ctx, m_factory->m_space_as_selection ? 1 : 0 );
 	// SCIM_CONFIG_IMENGINE_CHEWING_ESC_CLEAN_ALL_BUFFER
-	config.bEscCleanAllBuf = m_factory->m_esc_clean_all_buffer ? 1 : 0;
-
-	//SetConfig( &da, &config );
-	chewing_Configure( ctx, &config );
+	chewing_set_escCleanAllBuf( ctx, m_factory->m_esc_clean_all_buffer ? 1 : 0 );
 }
 
 
 ChewingIMEngineInstance::~ChewingIMEngineInstance()
 {
-	chewing_free( ctx );
+	chewing_delete( ctx );
 	m_reload_signal_connection.disconnect();
 }
 
-bool ChewingIMEngineInstance::process_key_event( const KeyEvent& key )
+bool ChewingIMEngineInstance::process_key_event( const KeyEvent& rawkey )
 {
     SCIM_DEBUG_IMENGINE( 2 ) <<
         "Process Key Event\n";
 
+	KeyEvent key = rawkey.map_to_layout(SCIM_KEYBOARD_Default);
+
 	if ( match_key_event( m_factory->m_chi_eng_keys, key ) ) {
 		m_prev_key = key;
 		trigger_property( SCIM_PROP_CHI_ENG_MODE );
@@ -362,8 +362,8 @@ bool ChewingIMEngineInstance::process_key_event( const KeyEvent& key )
 	if ( key.is_key_release() ) {
         SCIM_DEBUG_IMENGINE( 2 ) <<
             "Key Release, End Process Key\n";
-		return true;
-    }
+		return false;
+	}
 
 	if ( key.mask == 0 ) {
 		switch ( key.code ) {
@@ -412,6 +412,9 @@ bool ChewingIMEngineInstance::process_key_event( const KeyEvent& key )
 			case SCIM_KEY_Page_Down:
 				chewing_handle_PageDown( ctx );
 				break;
+			case SCIM_KEY_KP_Enter:
+				chewing_handle_Enter( ctx );
+				break;
 			case SCIM_KEY_Shift_L:
 			case SCIM_KEY_Shift_R:
 			case SCIM_KEY_Control_L:
@@ -421,6 +424,46 @@ bool ChewingIMEngineInstance::process_key_event( const KeyEvent& key )
                 SCIM_DEBUG_IMENGINE( 2 ) <<
                     "Unused keys, End Process Key\n";
 				return true;
+			case SCIM_KEY_KP_0:
+			case SCIM_KEY_KP_1:
+			case SCIM_KEY_KP_2:
+			case SCIM_KEY_KP_3:
+			case SCIM_KEY_KP_4:
+			case SCIM_KEY_KP_5:
+			case SCIM_KEY_KP_6:
+			case SCIM_KEY_KP_7:
+			case SCIM_KEY_KP_8:
+			case SCIM_KEY_KP_9:
+			case SCIM_KEY_KP_Decimal:
+			case SCIM_KEY_KP_Add: 
+			case SCIM_KEY_KP_Subtract: 
+			case SCIM_KEY_KP_Multiply: 
+			case SCIM_KEY_KP_Divide: 
+				chewing_handle_Numlock( ctx,
+					'0' + key.code - SCIM_KEY_KP_0 );
+				break;
+			case SCIM_KEY_F1:
+			case SCIM_KEY_F2:
+			case SCIM_KEY_F3:
+			case SCIM_KEY_F4:
+			case SCIM_KEY_F5:
+			case SCIM_KEY_F6:
+			case SCIM_KEY_F7:
+			case SCIM_KEY_F8:
+			case SCIM_KEY_F9:
+			case SCIM_KEY_F10:
+			case SCIM_KEY_F11:
+			case SCIM_KEY_F12:
+			case SCIM_KEY_F13:
+			case SCIM_KEY_F14:
+			case SCIM_KEY_F15:
+			case SCIM_KEY_F16:
+			case SCIM_KEY_F17:
+			case SCIM_KEY_F18:
+			case SCIM_KEY_F19:
+				SCIM_DEBUG_IMENGINE( 2 ) << 
+					"Release function keys.\n"; 
+				return false;
 			default:
                 SCIM_DEBUG_IMENGINE( 2 ) <<
                     "Begin OnKeyDefault\n";
@@ -477,7 +520,7 @@ bool ChewingIMEngineInstance::process_key_event( const KeyEvent& key )
 	have_input = true;
     SCIM_DEBUG_IMENGINE( 2 ) <<
         "End Process Key\n";
-	return commit( ctx->output );
+	return commit( ctx );
 }
 
 void ChewingIMEngineInstance::move_preedit_caret( unsigned int pos )
@@ -487,40 +530,32 @@ void ChewingIMEngineInstance::move_preedit_caret( unsigned int pos )
 void ChewingIMEngineInstance::select_candidate ( unsigned int index )
 {
 	chewing_handle_Default( ctx, '1' + index );
-	commit( ctx->output );
+	commit( ctx );
 }
 
 void ChewingIMEngineInstance::update_lookup_table_page_size(
 		unsigned int page_size )
 {
-	//XXX should not directly access data member.
-	ctx->data->config.candPerPage = page_size;
-	m_lookup_table.set_page_size (page_size);
+	chewing_set_candPerPage( ctx, page_size );
+	m_lookup_table.set_page_size( page_size );
 }
 
 void ChewingIMEngineInstance::lookup_table_page_up()
 {
 	chewing_handle_Space( ctx );
-	commit( ctx->output );
+	commit( ctx );
 }
 
 void ChewingIMEngineInstance::lookup_table_page_down()
 {
 	chewing_handle_Space( ctx );
-	commit( ctx->output );
+	commit( ctx );
 }
 
 void ChewingIMEngineInstance::reset()
 {
 	chewing_Reset( ctx );
 
-	/* Configure PinYin input method.
-	   if the PinYin method type is a built-in type,
-	   the second parameter will be ignored.
-	 */
-	chewing_set_PinYinMethod(static_cast<PinYinMethodType>(m_factory->m_PinYinType),
-		m_factory->m_ExternPinYinPath.c_str());
-	
 	/* Configure Keyboard Type */
 	chewing_set_KBType( ctx, chewing_KBStr2Num( 
 				(char *) m_factory->m_KeyboardType.c_str() ));
@@ -535,14 +570,15 @@ void ChewingIMEngineInstance::reset()
 
 	/* Configure selection keys definition */
 	int i = 0;
+	int *selkey = new int[ m_factory->m_selection_keys_num ];
 	for (; m_factory->m_selection_keys[i] &&
-	       i <= m_factory->m_selection_keys_num; i++) {
-		config.selKey[i] = m_factory->m_selection_keys[i];
+	       i < m_factory->m_selection_keys_num; i++) {
+		selkey[i] = m_factory->m_selection_keys[i];
 	}
-	config.selKey[i] = '\0';
+	chewing_set_selKey( ctx, selkey, m_factory->m_selection_keys_num );
 	m_lookup_table.init( m_factory->m_selection_keys,
 	                     m_factory->m_selection_keys_num );
-
+    delete [] selkey;
 	/* Re-focus to clear preedit, to avoid gedit crash when select-all. */
 	focus_out();
 	focus_in();
@@ -561,7 +597,7 @@ void ChewingIMEngineInstance::focus_out()
 		"Focus Out\n";
 	if (have_input == true) {
 		chewing_handle_Enter( ctx );
-		commit( ctx->output );
+		commit( ctx );
 		chewing_handle_Esc( ctx ); 
 		have_input = false;
 	}
@@ -570,91 +606,84 @@ void ChewingIMEngineInstance::focus_out()
 void ChewingIMEngineInstance::trigger_property( const String& property )
 {
 	if ( property == SCIM_PROP_CHI_ENG_MODE ) {
-		commit( ctx->output );
+		commit( ctx );
 		chewing_set_ChiEngMode( ctx, !chewing_get_ChiEngMode ( ctx ) );
 	} else if ( property == SCIM_PROP_LETTER ) {
 		chewing_set_ShapeMode( ctx, !chewing_get_ShapeMode( ctx ) );
 	} else if ( property == SCIM_PROP_KBTYPE ) {
-		/* loop through the keyboard type array */
-		if ( chewing_get_KBType( ctx ) == KB_TYPE_NUM - 1 )
- 	                chewing_set_KBType( ctx, KB_DEFAULT );
-		else
-			chewing_set_KBType( ctx, chewing_get_KBType( ctx ) + 1 );
+		chewing_set_KBType( ctx, chewing_get_KBType( ctx ) + 1 );
 	}
 	refresh_all_properties ();
 }
 
-bool ChewingIMEngineInstance::commit( ChewingOutput *pgo )
+bool ChewingIMEngineInstance::commit( ChewingContext* ctx )
 {
 	AttributeList attr;
 
-    SCIM_DEBUG_IMENGINE( 2 ) <<
-        "IMEngine Instance Commit\n";
+	SCIM_DEBUG_IMENGINE( 2 ) << "IMEngine Instance Commit\n";
 	// commit string
-	m_commit_string = L"";
-	if ( pgo->keystrokeRtn & KEYSTROKE_COMMIT ) {
-		for ( int i = 0; i < pgo->nCommitStr; i++ ) {
-			m_commit_string += utf8_mbstowcs((char *)pgo->commitStr[ i ].s, 
-					MAX_UTF8_SIZE);
-            SCIM_DEBUG_IMENGINE( 2 ) << "Commit Add: " <<
-                (char *)pgo->commitStr[ i ].s << "\n";
+	if ( chewing_commit_Check( ctx ) ) {
+		char* str = chewing_commit_String(ctx);
+		if ( str ) {
+			commit_string( utf8_mbstowcs( str ) );
+			chewing_free( str );
 		}
-		commit_string( m_commit_string );
 	}
-	m_preedit_string = L"";
+
 	// preedit string
-	// XXX show Interval
-	for ( int i = 0; i < pgo->chiSymbolCursor; i++ ) {
-		m_preedit_string += utf8_mbstowcs((char *)pgo->chiSymbolBuf[ i ].s, MAX_UTF8_SIZE);
-		SCIM_DEBUG_IMENGINE( 2 ) << "PreEdit Add: " <<
-			(char *)pgo->chiSymbolBuf[ i ].s << "\n";
-	}
-	// zuin string
-	for ( int i = 0, j = 0; i < ZUIN_SIZE; i++ ) {
-		if ( pgo->zuinBuf[ i ].s[ 0 ] != '\0' ) {
-			m_preedit_string += utf8_mbstowcs((char *)pgo->zuinBuf[ i ].s, 
-					MAX_UTF8_SIZE);
-			attr.push_back(Attribute(pgo->chiSymbolCursor + j, 1,
-					SCIM_ATTR_DECORATE, SCIM_ATTR_DECORATE_REVERSE));
-			j++;
+	WideString preedit_string;
+	if( chewing_buffer_Check( ctx ) ) {
+		char* chibuf = chewing_buffer_String( ctx );
+		if ( chibuf ) {
+			preedit_string = utf8_mbstowcs( chibuf );
+			chewing_free( chibuf );
 		}
 	}
 
-	for ( int i = pgo->chiSymbolCursor; i < pgo->chiSymbolBufLen; i++ ) {
-        	m_preedit_string += utf8_mbstowcs((char *)pgo->chiSymbolBuf[ i ].s, 
-				MAX_UTF8_SIZE);
+	int zuin_count;
+	char* zuin_str = chewing_zuin_String( ctx, &zuin_count );
+	if( zuin_str ) {
+		preedit_string += utf8_mbstowcs( zuin_str );
+		chewing_free( zuin_str );
 	}
 
-	for ( int i = 0; i < pgo->nDispInterval; i++ ) {
-		if ( pgo->dispInterval[ i ].to - pgo->dispInterval[ i ].from > 1 ) {
+
+	chewing_interval_Enumerate( ctx );
+	int interval_count = 0;
+	while( chewing_interval_hasNext( ctx ) ) {
+		IntervalType it;
+		chewing_interval_Get( ctx, &it );
+		int len = it.to - it.from;
+		if( len > 1 ) {
 			attr.push_back(
-				Attribute(
-					pgo->dispInterval[ i ].from,
-					pgo->dispInterval[ i ].to - pgo->dispInterval[ i ].from,
+				Attribute( it.from, len,
 					SCIM_ATTR_DECORATE,
-					SCIM_ATTR_DECORATE_UNDERLINE));
+					SCIM_ATTR_DECORATE_UNDERLINE) );
 			attr.push_back(
-				Attribute(
-					pgo->dispInterval[ i ].from,
-					pgo->dispInterval[ i ].to - pgo->dispInterval[ i ].from,
+				Attribute( it.from, len,
 					SCIM_ATTR_BACKGROUND,
-	m_factory->m_preedit_bgcolor[i % SCIM_CONFIG_IMENGINE_CHEWING_PREEDIT_BGCOLOR_NUM] ));
+					m_factory->m_preedit_bgcolor[interval_count % SCIM_CONFIG_IMENGINE_CHEWING_PREEDIT_BGCOLOR_NUM] ) );
+				
 		}
+		++interval_count;
 	}
+
 	// cursor decoration
-	if ( pgo->zuinBuf[ 0 ].s[ 0 ] == '\0' )
+	int current_cursor = chewing_cursor_Current( ctx );
+	if( chewing_zuin_Check( ctx ) ) {
 		attr.push_back(
 			Attribute(
-				pgo->chiSymbolCursor, 
+				current_cursor,
 				1, 
 				SCIM_ATTR_DECORATE, SCIM_ATTR_DECORATE_REVERSE));
+	}
 
 	// update display
-	update_preedit_string( m_preedit_string, attr );
-	update_preedit_caret( pgo->chiSymbolCursor );
+	update_preedit_string( preedit_string, attr );
+	update_preedit_caret( current_cursor );
 
 	// show preedit string
-	if ( m_preedit_string.empty() ) {
+	if ( preedit_string.empty() ) {
 		hide_preedit_string();
 	}
 	else {
@@ -662,43 +691,43 @@ bool ChewingIMEngineInstance::commit( ChewingOutput *pgo )
 	}
 	
 	// show lookup table
-        if ( !pgo->pci )
-                return true;
-	if ( pgo->pci->nPage != 0 ) {
-		m_lookup_table.update( pgo->pci );
+	if( chewing_cand_CheckDone( ctx ) )
+		return true;
+
+	int total_pages = chewing_cand_TotalPage( ctx );
+	if( total_pages != 0 ) {
+		m_lookup_table.update( ctx );
 		show_lookup_table();
 
-		if ( ( pgo->pci->nTotalChoice % pgo->pci->nChoicePerPage
-					!= 0 ) && pgo->pci->pageNo == pgo->pci->nPage - 1 ) {
-			m_lookup_table.set_page_size(
-					pgo->pci->nTotalChoice %
-					pgo->pci->nChoicePerPage );
-		} else {
-			m_lookup_table.set_page_size(
-					pgo->pci->nChoicePerPage );
-		}
+		int choice_per_page = chewing_cand_ChoicePerPage( ctx );
+		if( chewing_cand_CurrentPage( ctx ) < total_pages )
+			m_lookup_table.set_page_size( choice_per_page );
+		else
+			m_lookup_table.set_page_size( chewing_cand_TotalChoice( ctx ) % choice_per_page );
+
 		update_lookup_table( m_lookup_table );
 	}
 	else {
 		hide_lookup_table();
 	}
+
 	
 	// show aux string
-	m_aux_string = L"";
-	if ( pgo->bShowMsg ) {
-		for ( int i = 0; i < pgo->showMsgLen; i++ ) {
-            m_aux_string += utf8_mbstowcs((char *)pgo->showMsg[ i ].s, MAX_UTF8_SIZE);
+	if( chewing_aux_Check( ctx ) ) {
+		char* aux_str = chewing_aux_String( ctx );
+		if( aux_str ) {
+			update_aux_string( utf8_mbstowcs( aux_str ) );
+			chewing_free( aux_str );
+			show_aux_string();
 		}
-		update_aux_string( m_aux_string );
-		show_aux_string();
-		pgo->showMsgLen = 0;
 	}
 	else {
 		hide_aux_string();
 	}
-	if ( pgo->keystrokeRtn & KEYSTROKE_ABSORB )
+
+	if( chewing_keystroke_CheckAbsorb( ctx ) )
 		return true;
-	if ( pgo->keystrokeRtn & KEYSTROKE_IGNORE )
+	if( chewing_keystroke_CheckIgnore( ctx ) )
 		return false;
 	return true;
 }
@@ -747,39 +776,28 @@ void ChewingIMEngineInstance::refresh_chieng_property ()
 void ChewingIMEngineInstance::refresh_kbtype_property ()
 {
 	/* look forward to be evoluted into drop-down list */
-	switch ( chewing_get_KBType( ctx ) )
-	{
-		case KB_DEFAULT:
-			_kbtype_property.set_label (_("Default"));
-			break;
-		case KB_HSU:
-			_kbtype_property.set_label (_("Hsu's"));
-			break;
-		case KB_IBM:
-			_kbtype_property.set_label (_("IBM"));
-			break;
-		case KB_GIN_YIEH:
-			_kbtype_property.set_label (_("Gin-Yieh"));
-			break;
-		case KB_ET:
-			_kbtype_property.set_label (_("ETen"));
-			break;
-		case KB_ET26:
-			_kbtype_property.set_label (_("ETen 26-key"));
-			break;
-		case KB_DVORAK:
-			_kbtype_property.set_label (_("Dvorak"));
-			break;
-		case KB_DVORAK_HSU:
-			_kbtype_property.set_label (_("Dvorak Hsu's"));
-			break;
-		case KB_HANYU_PINYIN:
-			_kbtype_property.set_label (_("Han-Yu"));
-			break;
-		default:
-			_kbtype_property.set_label (_("Default"));
-		}
-
+	char *s = chewing_get_KBString( ctx );
+	if ( ! strcmp( s, "KB_DEFAULT" ) )
+		_kbtype_property.set_label (_("Default"));
+	else if ( ! strcmp( s, "KB_HSU" ) )
+		_kbtype_property.set_label (_("Hsu's"));
+	else if ( ! strcmp( s, "KB_IBM" ) )
+		_kbtype_property.set_label (_("IBM"));
+	else if ( ! strcmp( s, "KB_GIN_YIEH" ) )
+		_kbtype_property.set_label (_("Gin-Yieh"));
+	else if ( ! strcmp( s, "KB_ET" ) )
+		_kbtype_property.set_label (_("ETen"));
+	else if ( ! strcmp( s, "KB_ET26" ) )
+		_kbtype_property.set_label (_("ETen 26-key"));
+	else if ( ! strcmp( s, "KB_DVORAK" ) )
+		_kbtype_property.set_label (_("Dvorak"));
+	else if ( ! strcmp( s, "KB_DVORAK_HSU" ) )
+		_kbtype_property.set_label (_("Dvorak Hsu's"));
+	else if ( ! strcmp( s, "KB_PINYIN" ) )
+		_kbtype_property.set_label (_("Han-Yu"));
+	else
+		_kbtype_property.set_label (_("Default"));
+	chewing_free( s );
 	update_property (_kbtype_property);
 }
 
@@ -804,12 +822,18 @@ ChewingLookupTable::~ChewingLookupTable()
 
 WideString ChewingLookupTable::get_candidate( int index ) const
 {
-	WideString m_converted_string;
-	int no = pci->pageNo * pci->nChoicePerPage;
-	m_converted_string = utf8_mbstowcs(
-			(char *) pci->totalChoiceStr[ no + index ],
-			(int) strlen( (char *) pci->totalChoiceStr[ no + index ] ));
-	return m_converted_string;
+	if( index == 0 )
+		chewing_cand_Enumerate( ctx );
+
+	WideString converted_string;
+	if( chewing_cand_hasNext( ctx ) ) {
+		char* str = chewing_cand_String( ctx );
+		if( str ) {
+			converted_string = utf8_mbstowcs( str );
+			chewing_free( str );
+		}
+	}
+	return converted_string;
 }
 
 AttributeList ChewingLookupTable::get_attributes( int index ) const
@@ -819,7 +843,7 @@ AttributeList ChewingLookupTable::get_attributes( int index ) const
 
 unsigned int ChewingLookupTable::number_of_candidates() const
 {
-	return pci->nTotalChoice;
+	return chewing_cand_TotalChoice( ctx );
 }
 
 void ChewingLookupTable::clear()
@@ -840,8 +864,7 @@ void ChewingLookupTable::init(String selection_keys, int selection_keys_num)
 	set_candidate_labels( labels );
 }
 
-void ChewingLookupTable::update( ChoiceInfo *ci )
+void ChewingLookupTable::update( ChewingContext* ctx )
 {
-	pci = ci;
+	this->ctx = ctx;
 }
-
