@@ -6,7 +6,7 @@
  * SCIM-chewing -
  *	Intelligent Chinese Phonetic IM Engine for SCIM.
  *
- * Copyright (c) 2005, 2006
+ * Copyright (c) 2005, 2006, 2008
  *	SCIM-chewing Developers. See ChangeLog for details.
  *
  * See the file "COPYING" for information on usage and redistribution
@@ -89,7 +89,7 @@ extern "C" {
 
 	String scim_setup_module_get_description()
 	{
-		return String( _( "A Intelligent Chinese Phonetic IMEngine Module." ) );
+		return String( _( "An Intelligent Chinese Phonetic IMEngine Module." ) );
 	}
 
 	void scim_setup_module_load_config( const ConfigPointer &config )
@@ -135,13 +135,13 @@ struct ColorConfigData {
 // Internal data declaration.
 // static bool __config_use_capslock          = true;
 static bool __config_add_phrase_forward = false;
+static bool __config_phrase_choice_rearward = true;
+static bool __config_auto_shift_cursor = true;
 static bool __config_esc_clean_all_buffer = false;
 static bool __config_space_as_selection = true;
 // static bool __config_show_candidate_comment= true;
 static String __config_kb_type_data;
 static String __config_kb_type_data_translated;
-static int __config_pinyin_type_data = 0;
-static String __config_external_pinyin_path;
 static String __config_selKey_type_data;
 static String __config_selKey_num_data;
 static String __config_chieng_mode_data;
@@ -149,6 +149,8 @@ static bool __have_changed                 = false;
 
 // static GtkWidget    * __widget_use_capslock          = 0;
 static GtkWidget    * __widget_add_phrase_forward = 0;
+static GtkWidget    * __widget_phrase_choice_rearward = 0;
+static GtkWidget    * __widget_auto_shift_cursor = 0;
 static GtkWidget    * __widget_esc_clean_all_buffer = 0;
 static GtkWidget    * __widget_space_as_selection = 0;
 static GtkWidget    * __widget_kb_type = 0;
@@ -182,7 +184,7 @@ static KeyboardConfigData __config_keyboards[] =
         // button
         NULL,
         // data
-        "Ctrl+space"
+        "Control+space"
     },
     {
         // key
@@ -309,19 +311,6 @@ static void on_color_button_changed(
 	ScimChewingColorButton *button,
 	gpointer user_data );
 
-static void on_builtin_pinyin_method_changed(
-	GtkEditable *editable,
-	gpointer user_data );
-	
-
-static void on_builtin_pinyin_method_toggled(
-	GtkButton *button,
-	gpointer user_data );
-
-static void on_external_pinyin_method_changed(
-	GtkWidget *widget,
-	gpointer user_data );
-
 static void setup_widget_value();
 
 static GtkWidget *create_options_page();
@@ -351,6 +340,36 @@ static GtkWidget *create_options_page()
 			__widget_tooltips, __widget_add_phrase_forward,
 			_( "Whether to add Phrase forward or not." ), NULL );
 
+	__widget_phrase_choice_rearward =
+		gtk_check_button_new_with_mnemonic( _( "_Rearward phrase choice" ) );
+	gtk_widget_show( __widget_phrase_choice_rearward );
+	gtk_box_pack_start( GTK_BOX( vbox ), __widget_phrase_choice_rearward, FALSE, FALSE, 4 );
+	gtk_container_set_border_width( GTK_CONTAINER( __widget_phrase_choice_rearward ), 4 );
+
+	g_signal_connect(
+			(gpointer) __widget_phrase_choice_rearward, "toggled",
+			G_CALLBACK( on_default_toggle_button_toggled ),
+			&__config_phrase_choice_rearward );
+
+	gtk_tooltips_set_tip(
+			__widget_tooltips, __widget_phrase_choice_rearward,
+			_( "The behavior for phrase choice to be rearward or not." ), NULL );
+
+	__widget_auto_shift_cursor =
+		gtk_check_button_new_with_mnemonic( _( "_Automatically shift cursor" ) );
+	gtk_widget_show( __widget_auto_shift_cursor );
+	gtk_box_pack_start( GTK_BOX( vbox ), __widget_auto_shift_cursor, FALSE, FALSE, 4 );
+	gtk_container_set_border_width( GTK_CONTAINER( __widget_auto_shift_cursor ), 4 );
+
+	g_signal_connect(
+			(gpointer) __widget_auto_shift_cursor, "toggled",
+			G_CALLBACK( on_default_toggle_button_toggled ),
+			&__config_auto_shift_cursor );
+
+	gtk_tooltips_set_tip(
+			__widget_tooltips, __widget_auto_shift_cursor,
+			_( "Automatically shift cursor after selection." ), NULL );
+
 	__widget_esc_clean_all_buffer =
 		gtk_check_button_new_with_mnemonic(_( "_Esc key to clean all buffer" ) );
 	gtk_widget_show( __widget_esc_clean_all_buffer );
@@ -394,6 +413,9 @@ struct _builtin_keymap {
 		{       
 			"KB_HSU",
 			String( _( "Hsu's Keyboard" ) ) },
+		{
+			"KB_IBM",
+			String( _( "IBM Keyboard" ) ) },
 		{       
 			"KB_GIN_YEIH",
 			String( _( "Gin-Yieh Keyboard" ) ) },
@@ -409,16 +431,14 @@ struct _builtin_keymap {
 		{
 			"KB_DVORAK_HSU",
 			String( _( "Dvorak Keyboard with Hsu's support" ) ) },
+		{
+			"KB_DACHEN_CP26",
+			String( _( "DACHEN_CP26 Keyboard") ) },
 		{
 			"KB_HANYU_PINYIN",
 			String( _( "Han-Yu PinYin Keyboard" ) ) }
 };
 
-String _builtin_pinyin_map[] = {
-	String( _("Han-Yu PinYin") )
-};
-	
-
 static const char *builtin_selectkeys[] = {
 	SCIM_CONFIG_IMENGINE_CHEWING_SELECTION_KEYS,
 	"asdfghjkl;",
@@ -447,10 +467,10 @@ static GtkWidget *create_keyboard_page()
 	GtkWidget *table;
 	GtkWidget *label;
 
-	table = gtk_table_new (5, 5, FALSE);
+	table = gtk_table_new (6, 3, FALSE);
 	gtk_widget_show (table);
 
-	int i;
+	size_t i;
 	// Create keyboard setting.
 	for (i = 0; __config_keyboards [i].key; ++ i) {
 		label = gtk_label_new (NULL);
@@ -477,143 +497,6 @@ static GtkWidget *create_keyboard_page()
 		gtk_label_set_mnemonic_widget (GTK_LABEL (label), __config_keyboards[i].button);
 	}
 
-	// Setup KB_TYPE combo box
-	__widget_kb_type = gtk_combo_new();
-	gtk_widget_show (__widget_kb_type);
-
-	for (
-		i = 0; 
-		i < (int) (sizeof(builtin_keymaps) / sizeof(_builtin_keymap)); 
-		i++) {
-		kb_type_list = g_list_append(
-				kb_type_list,
-				(void *) builtin_keymaps[ i ].translated_name.c_str() );
-	}
-	
-	gtk_combo_set_popdown_strings (GTK_COMBO (__widget_kb_type), kb_type_list);
-	g_list_free(kb_type_list);
-	gtk_combo_set_use_arrows (GTK_COMBO (__widget_kb_type), TRUE);
-	gtk_editable_set_editable (GTK_EDITABLE (GTK_ENTRY (GTK_COMBO (__widget_kb_type)->entry)), FALSE);
-	label = gtk_label_new (_("Use keyboard type:"));
-	gtk_widget_show (label);
-	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5,
-			(GtkAttachOptions) (GTK_FILL),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_table_attach (GTK_TABLE (table), __widget_kb_type, 1, 2, 4, 5,
-			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_tooltips_set_tip (__widget_tooltips, GTK_COMBO (__widget_kb_type)->entry,
-			_("Change the default keyboard layout type"), NULL);
-	g_signal_connect(
-		(gpointer) GTK_ENTRY(GTK_COMBO(__widget_kb_type)->entry), 
-		"changed",
-		G_CALLBACK (on_default_editable_changed),
-		&(__config_kb_type_data_translated));
-
-	// Setup chieng_mode combo box
-	__widget_chieng_mode = gtk_combo_new();
-	gtk_widget_show (__widget_chieng_mode);
-
-	for (i = 0; 
-	     i < (sizeof(builtin_chieng_mode) / sizeof(builtin_chieng_mode[0])); 
-	     i++) {
-		chieng_mode_list = g_list_append(
-				chieng_mode_list,
-				(void *) builtin_chieng_mode[ i ] );
-	}
-	
-	gtk_combo_set_popdown_strings (GTK_COMBO (__widget_chieng_mode), chieng_mode_list);
-	g_list_free(chieng_mode_list);
-	gtk_combo_set_use_arrows (GTK_COMBO (__widget_chieng_mode), TRUE);
-	gtk_editable_set_editable (GTK_EDITABLE (GTK_ENTRY (GTK_COMBO (__widget_chieng_mode)->entry)), FALSE);
-	label = gtk_label_new (_("Initial trigger Chinese/English mode:"));
-	gtk_widget_show (label);
-	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i+1,
-			(GtkAttachOptions) (GTK_FILL),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_table_attach (GTK_TABLE (table), __widget_chieng_mode, 1, 2, i, i+1,
-			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_tooltips_set_tip (__widget_tooltips, GTK_COMBO (__widget_chieng_mode)->entry,
-			_("Change the default Chinese/English mode on every trigger"), NULL);
-	g_signal_connect(
-		(gpointer) GTK_ENTRY(GTK_COMBO(__widget_chieng_mode)->entry), 
-		"changed",
-		G_CALLBACK (on_default_editable_changed),
-		&(__config_chieng_mode_data));
-
-	// Setup selKey combo box
-	__widget_selKey_type = gtk_combo_new();
-	gtk_widget_show (__widget_selKey_type);
-
-	for (i = 0; 
-	     i < (sizeof(builtin_selectkeys) / sizeof(builtin_selectkeys[0])); 
-	     i++) {
-		selKey_type_list = g_list_append(
-				selKey_type_list,
-				(void *) builtin_selectkeys[ i ] );
-	}
-	
-	gtk_combo_set_popdown_strings (GTK_COMBO (__widget_selKey_type), selKey_type_list);
-	g_list_free(selKey_type_list);
-	gtk_combo_set_use_arrows (GTK_COMBO (__widget_selKey_type), TRUE);
-	gtk_editable_set_editable (GTK_EDITABLE (GTK_ENTRY (GTK_COMBO (__widget_selKey_type)->entry)), FALSE);
-	label = gtk_label_new (_("Customized Selection Keys:"));
-	gtk_widget_show (label);
-	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
-			(GtkAttachOptions) (GTK_FILL),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_table_attach (GTK_TABLE (table), __widget_selKey_type, 1, 2, 3, 4,
-			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_tooltips_set_tip (__widget_tooltips, GTK_COMBO (__widget_selKey_type)->entry,
-			_("Change the default selection keys"), NULL);
-	g_signal_connect(
-		(gpointer) GTK_ENTRY(GTK_COMBO(__widget_selKey_type)->entry), 
-		"changed",
-		G_CALLBACK (on_default_editable_changed),
-		&(__config_selKey_type_data));
-
-	// Setup selKey_num combo box
-	__widget_selKey_num = gtk_combo_new();
-	gtk_widget_show (__widget_selKey_num);
-
-	for (i = 0; 
-	     i < (sizeof(builtin_selectkeys_num) / sizeof(builtin_selectkeys_num[0])); 
-	     i++) {
-		selKey_num_list = g_list_append(
-				selKey_num_list,
-				(void *) builtin_selectkeys_num[ i ] );
-	}
-	
-	gtk_combo_set_popdown_strings (GTK_COMBO (__widget_selKey_num), selKey_num_list);
-	g_list_free(selKey_num_list);
-	gtk_combo_set_use_arrows (GTK_COMBO (__widget_selKey_num), TRUE);
-	gtk_editable_set_editable (GTK_EDITABLE (GTK_ENTRY (GTK_COMBO (__widget_selKey_num)->entry)), FALSE);
-	label = gtk_label_new (_("Number of Selection Keys :"));
-	gtk_widget_show (label);
-	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
-			(GtkAttachOptions) (GTK_FILL),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_table_attach (GTK_TABLE (table), __widget_selKey_num, 1, 2, 2, 3,
-			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_tooltips_set_tip (__widget_tooltips, GTK_COMBO (__widget_selKey_num)->entry,
-			_("Change the default number of selection keys"), NULL);
-	g_signal_connect(
-		(gpointer) GTK_ENTRY(GTK_COMBO(__widget_selKey_num)->entry), 
-		"changed",
-		G_CALLBACK (on_default_editable_changed),
-		&(__config_selKey_num_data));
-
 	// keyboard: trigger keys
 	for (i = 0; __config_keyboards [i].key; ++ i) {
 		g_signal_connect ((gpointer) __config_keyboards [i].button, "clicked",
@@ -629,6 +512,142 @@ static GtkWidget *create_keyboard_page()
 				_(__config_keyboards [i].tooltip), NULL);
 	}
 
+	// Setup chieng_mode combo box
+	__widget_chieng_mode = gtk_combo_new();
+	gtk_widget_show (__widget_chieng_mode);
+
+	for (i = 0; 
+			i < (sizeof(builtin_chieng_mode) / sizeof(builtin_chieng_mode[0])); 
+			i++) {
+		chieng_mode_list = g_list_append(
+				chieng_mode_list,
+				(void *) builtin_chieng_mode[ i ] );
+	}
+
+	gtk_combo_set_popdown_strings (GTK_COMBO (__widget_chieng_mode), chieng_mode_list);
+	g_list_free(chieng_mode_list);
+	gtk_combo_set_use_arrows (GTK_COMBO (__widget_chieng_mode), TRUE);
+	gtk_editable_set_editable (GTK_EDITABLE (GTK_ENTRY (GTK_COMBO (__widget_chieng_mode)->entry)), FALSE);
+	label = gtk_label_new (_("Initial trigger Chinese/English mode:"));
+	gtk_widget_show (label);
+	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
+			(GtkAttachOptions) (GTK_FILL),
+			(GtkAttachOptions) (GTK_FILL), 4, 4);
+	gtk_table_attach (GTK_TABLE (table), __widget_chieng_mode, 1, 2, 2, 3,
+			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+			(GtkAttachOptions) (GTK_FILL), 4, 4);
+	gtk_tooltips_set_tip (__widget_tooltips, GTK_COMBO (__widget_chieng_mode)->entry,
+			_("Change the default Chinese/English mode on every trigger"), NULL);
+	g_signal_connect(
+			(gpointer) GTK_ENTRY(GTK_COMBO(__widget_chieng_mode)->entry), 
+			"changed",
+			G_CALLBACK (on_default_editable_changed),
+			&(__config_chieng_mode_data));
+
+	// Setup selKey_num combo box
+	__widget_selKey_num = gtk_combo_new();
+	gtk_widget_show (__widget_selKey_num);
+
+	for (i = 0; 
+	     i < (sizeof(builtin_selectkeys_num) / sizeof(builtin_selectkeys_num[0])); 
+	     i++) {
+		selKey_num_list = g_list_append(
+				selKey_num_list,
+				(void *) builtin_selectkeys_num[ i ] );
+	}
+
+	gtk_combo_set_popdown_strings (GTK_COMBO (__widget_selKey_num), selKey_num_list);
+	g_list_free(selKey_num_list);
+	gtk_combo_set_use_arrows (GTK_COMBO (__widget_selKey_num), TRUE);
+	gtk_editable_set_editable (GTK_EDITABLE (GTK_ENTRY (GTK_COMBO (__widget_selKey_num)->entry)), FALSE);
+	label = gtk_label_new (_("Number of Selection Keys :"));
+	gtk_widget_show (label);
+	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
+			(GtkAttachOptions) (GTK_FILL),
+			(GtkAttachOptions) (GTK_FILL), 4, 4);
+	gtk_table_attach (GTK_TABLE (table), __widget_selKey_num, 1, 2, 3, 4,
+			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+			(GtkAttachOptions) (GTK_FILL), 4, 4);
+	gtk_tooltips_set_tip (__widget_tooltips, GTK_COMBO (__widget_selKey_num)->entry,
+			_("Change the default number of selection keys"), NULL);
+	g_signal_connect(
+			(gpointer) GTK_ENTRY(GTK_COMBO(__widget_selKey_num)->entry), 
+			"changed",
+			G_CALLBACK (on_default_editable_changed),
+			&(__config_selKey_num_data));
+
+	// Setup selKey combo box
+	__widget_selKey_type = gtk_combo_new();
+	gtk_widget_show (__widget_selKey_type);
+
+	for (i = 0;
+	     i < (sizeof(builtin_selectkeys) / sizeof(builtin_selectkeys[0]));
+	     i++) {
+		selKey_type_list = g_list_append(
+				selKey_type_list,
+				(void *) builtin_selectkeys[ i ] );
+	}
+
+	gtk_combo_set_popdown_strings (GTK_COMBO (__widget_selKey_type), selKey_type_list);
+	g_list_free(selKey_type_list);
+	gtk_combo_set_use_arrows (GTK_COMBO (__widget_selKey_type), TRUE);
+	gtk_editable_set_editable (GTK_EDITABLE (GTK_ENTRY (GTK_COMBO (__widget_selKey_type)->entry)), FALSE);
+	label = gtk_label_new (_("Customized Selection Keys:"));
+	gtk_widget_show (label);
+	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5,
+			(GtkAttachOptions) (GTK_FILL),
+			(GtkAttachOptions) (GTK_FILL), 4, 4);
+	gtk_table_attach (GTK_TABLE (table), __widget_selKey_type, 1, 2, 4, 5,
+			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+			(GtkAttachOptions) (GTK_FILL), 4, 4);
+	gtk_tooltips_set_tip (__widget_tooltips, GTK_COMBO (__widget_selKey_type)->entry,
+			_("Change the default selection keys"), NULL);
+	g_signal_connect(
+			(gpointer) GTK_ENTRY(GTK_COMBO(__widget_selKey_type)->entry),
+			"changed",
+			G_CALLBACK (on_default_editable_changed),
+			&(__config_selKey_type_data));
+
+	// Setup KB_TYPE combo box
+	__widget_kb_type = gtk_combo_new();
+	gtk_widget_show (__widget_kb_type);
+
+	for (i = 0;
+	     i < (int) (sizeof(builtin_keymaps) / sizeof(_builtin_keymap));
+	     i++) {
+		kb_type_list = g_list_append(
+				kb_type_list,
+				(void *) builtin_keymaps[ i ].translated_name.c_str() );
+	}
+
+	gtk_combo_set_popdown_strings (GTK_COMBO (__widget_kb_type), kb_type_list);
+	g_list_free(kb_type_list);
+	gtk_combo_set_use_arrows (GTK_COMBO (__widget_kb_type), TRUE);
+	gtk_editable_set_editable (GTK_EDITABLE (GTK_ENTRY (GTK_COMBO (__widget_kb_type)->entry)), FALSE);
+	label = gtk_label_new (_("Use keyboard type:"));
+	gtk_widget_show (label);
+	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+	gtk_misc_set_padding (GTK_MISC (label), 4, 0);
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 5, 6,
+			(GtkAttachOptions) (GTK_FILL),
+			(GtkAttachOptions) (GTK_FILL), 4, 4);
+	gtk_table_attach (GTK_TABLE (table), __widget_kb_type, 1, 2, 5, 6,
+			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+			(GtkAttachOptions) (GTK_FILL), 4, 4);
+	gtk_tooltips_set_tip (__widget_tooltips, GTK_COMBO (__widget_kb_type)->entry,
+			_("Change the default keyboard layout type"), NULL);
+	g_signal_connect(
+			(gpointer) GTK_ENTRY(GTK_COMBO(__widget_kb_type)->entry),
+			"changed",
+			G_CALLBACK (on_default_editable_changed),
+			&(__config_kb_type_data_translated));
+
 	return table;
 }
 
@@ -656,71 +675,6 @@ static GtkWidget *create_color_button_page()
 	return table;
 }
 
-static GtkWidget *create_pinyin_config_page()
-{
-	GtkWidget *table;
-	GtkWidget *label;
-
-	table = gtk_table_new (4, 5, FALSE);
-
-	int i = 0;
-
-	// Setup KB_TYPE combo box
-	GtkWidget* widget_pinyin_type = gtk_combo_new();
-	gtk_widget_show (widget_pinyin_type);
-	GList* pinyin_type_list = NULL;
-
-	for (int i = 0; i < (int) sizeof(_builtin_pinyin_map)/sizeof(_builtin_pinyin_map[0]); ++i) {
-		pinyin_type_list = g_list_append(
-			pinyin_type_list, (void*) _builtin_pinyin_map[i].c_str());
-	}
-
-	gtk_combo_set_popdown_strings (GTK_COMBO (widget_pinyin_type), pinyin_type_list);
-	g_list_free(pinyin_type_list);
-	gtk_combo_set_use_arrows (GTK_COMBO (widget_pinyin_type), TRUE);
-	gtk_editable_set_editable (GTK_EDITABLE (GTK_ENTRY (GTK_COMBO (widget_pinyin_type)->entry)), FALSE);
-	g_signal_connect(
-		(gpointer) GTK_ENTRY(GTK_COMBO(widget_pinyin_type)->entry), 
-		"changed",
-		G_CALLBACK (on_builtin_pinyin_method_changed),
-		NULL);
-
-	GtkWidget* dialog = gtk_file_chooser_dialog_new(_("Select a file"),
-		NULL,
-		GTK_FILE_CHOOSER_ACTION_OPEN,
-		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-		NULL);
-	__file_btn = gtk_file_chooser_button_new_with_dialog(dialog);
-	g_signal_connect(G_OBJECT(dialog), "response",
-		G_CALLBACK(on_external_pinyin_method_changed), NULL);
-
-	__radio_builtin = gtk_radio_button_new_with_label(NULL, _("Built-in Method:"));
-	g_signal_connect(G_OBJECT(__radio_builtin), "toggled",
-		G_CALLBACK(on_builtin_pinyin_method_toggled),
-		GTK_COMBO(widget_pinyin_type)->entry);
-	
-	__radio_external = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(__radio_builtin), _("External Method:"));
-	g_signal_connect(G_OBJECT(__radio_external), "toggled",
-		G_CALLBACK(on_external_pinyin_method_changed), NULL);
-	
-	gtk_table_attach (GTK_TABLE (table), __radio_builtin, 0, 1, 0, 1,
-			(GtkAttachOptions) (GTK_FILL),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_table_attach (GTK_TABLE (table), widget_pinyin_type, 1, 2, 0, 1,
-			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_table_attach (GTK_TABLE (table), __radio_external, 0, 1, 1, 2,
-			(GtkAttachOptions) (GTK_FILL),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-	gtk_table_attach (GTK_TABLE (table), __file_btn, 1, 2, 1, 2,
-			(GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-			(GtkAttachOptions) (GTK_FILL), 4, 4);
-
-	gtk_widget_show_all (table);
-	return table;
-}
-
 static GtkWidget *create_setup_window()
 {
 	static GtkWidget *window = 0;
@@ -764,15 +718,6 @@ static GtkWidget *create_setup_window()
 		gtk_widget_show (label);
 		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
 
-		// Create PinYin configuration page.
-		page = create_pinyin_config_page ();
-
-		// Create the label for this note page.
-		label = gtk_label_new (_("PinYin Configuration"));
-		gtk_widget_show (label);
-		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
-		
-
 		window = notebook;
 
 		setup_widget_value ();
@@ -789,6 +734,18 @@ void setup_widget_value()
 				__config_add_phrase_forward);
 	}
 
+	if ( __widget_phrase_choice_rearward ) {
+		gtk_toggle_button_set_active(
+				GTK_TOGGLE_BUTTON( __widget_phrase_choice_rearward ),
+				__config_phrase_choice_rearward );
+	}
+
+	if ( __widget_auto_shift_cursor ) {
+		gtk_toggle_button_set_active(
+				GTK_TOGGLE_BUTTON( __widget_auto_shift_cursor ),
+				__config_auto_shift_cursor );
+	}
+
 	if ( __widget_space_as_selection ) {
 		gtk_toggle_button_set_active(
 				GTK_TOGGLE_BUTTON( __widget_space_as_selection ),
@@ -893,34 +850,26 @@ void load_config( const ConfigPointer &config )
 		__config_add_phrase_forward =
 			config->read( String( SCIM_CONFIG_IMENGINE_CHEWING_ADD_PHRASE_FORWARD ),
 				__config_add_phrase_forward );
-		
+
+		__config_phrase_choice_rearward =
+			config->read( String( SCIM_CONFIG_IMENGINE_CHEWING_PHRASE_CHOICE_REARWARD ),
+				__config_phrase_choice_rearward );
+
+		__config_auto_shift_cursor =
+			config->read( String( SCIM_CONFIG_IMENGINE_CHEWING_AUTO_SHIFT_CURSOR ),
+				__config_auto_shift_cursor );
+
 		__config_esc_clean_all_buffer =
 			config->read( String( SCIM_CONFIG_IMENGINE_CHEWING_ESC_CLEAN_ALL_BUFFER ),
 				__config_esc_clean_all_buffer );
-		
+
 		__config_space_as_selection =
 			config->read( String( SCIM_CONFIG_IMENGINE_CHEWING_SPACE_AS_SELECTION ),
 				__config_space_as_selection );
-		
+
 		__config_kb_type_data = 
 			config->read( String( SCIM_CONFIG_IMENGINE_CHEWING_USER_KB_TYPE ),
 				__config_kb_type_data);
-		
-		__config_pinyin_type_data =
-			config->read( String( SCIM_CONFIG_IMENGINE_CHEWING_PINYIN_METHD ),
-				__config_pinyin_type_data);
-
-		if (__config_pinyin_type_data == PINYIN_EXTERNAL)
-			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(__radio_external), TRUE);
-		else
-			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(__radio_builtin),TRUE);
-		
-		__config_external_pinyin_path =
-			config->read( String( SCIM_CONFIG_IMENGINE_CHEWING_EXTERNAL_PINYIN_PATH),
-				__config_external_pinyin_path);
-
-		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(__file_btn),
-			__config_external_pinyin_path.c_str());
 
 		__config_selKey_type_data =
 			config->read( String( SCIM_CONFIG_IMENGINE_CHEWING_USER_SELECTION_KEYS ),
@@ -959,6 +908,12 @@ void save_config( const ConfigPointer &config )
 		config->write( String( SCIM_CONFIG_IMENGINE_CHEWING_ADD_PHRASE_FORWARD ),
 				__config_add_phrase_forward );
 
+		config->write( String( SCIM_CONFIG_IMENGINE_CHEWING_PHRASE_CHOICE_REARWARD ),
+				__config_phrase_choice_rearward );
+
+		config->write( String( SCIM_CONFIG_IMENGINE_CHEWING_AUTO_SHIFT_CURSOR ),
+				__config_auto_shift_cursor );
+
 		config->write( String( SCIM_CONFIG_IMENGINE_CHEWING_ESC_CLEAN_ALL_BUFFER ),
 				__config_esc_clean_all_buffer );
 
@@ -981,12 +936,6 @@ void save_config( const ConfigPointer &config )
 		config->write (String (SCIM_CONFIG_IMENGINE_CHEWING_USER_KB_TYPE),
 			__config_kb_type_data);
 
-		config->write (String (SCIM_CONFIG_IMENGINE_CHEWING_PINYIN_METHD),
-			__config_pinyin_type_data);
-
-		config->write (String (SCIM_CONFIG_IMENGINE_CHEWING_EXTERNAL_PINYIN_PATH),
-			__config_external_pinyin_path);
-
 		// SCIM_CONFIG_IMENGINE_CHEWING_USER_SELECTION_KEYS
 		int index_selectkeys =
 			sizeof(builtin_selectkeys) / sizeof(builtin_selectkeys[0]) - 1;
@@ -1185,42 +1134,3 @@ static void on_color_button_changed(
 		__have_changed = true;
 	}
 }
-
-static void set_pinyin_type(const String& str)
-{
-	for (int i = 0; i < (int) sizeof(_builtin_pinyin_map)/sizeof(_builtin_pinyin_map[0]); ++i) {
-		if ( str == _builtin_pinyin_map[i] ) {
-			__config_pinyin_type_data = i;
-			__have_changed = true;
-			return;
-		}
-	}
-}
-static void on_builtin_pinyin_method_changed (
-	GtkEditable *editable,
-	gpointer user_data )
-{
-	String str = String( gtk_entry_get_text( GTK_ENTRY( editable ) ) );
-	set_pinyin_type(str);
-}
-
-static void on_builtin_pinyin_method_toggled (
-	GtkButton *button,
-	gpointer user_data )
-{
-	GtkEditable* editable = static_cast <GtkEditable*> (user_data);
-	String str = String( gtk_entry_get_text( GTK_ENTRY( editable ) ) );
-	set_pinyin_type(str);	
-}
-
-static void on_external_pinyin_method_changed (
-	GtkWidget* widget, gpointer user_data)
-{
-	GSList* file_list = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(__file_btn));
-	if (file_list && file_list->data)
-		__config_external_pinyin_path = static_cast<char*>( file_list->data );
-
-	g_slist_free(file_list);
-	__config_pinyin_type_data = PINYIN_EXTERNAL;
-	__have_changed = true;		
-}
